const listeners = [];
const statics = {
  nowDepth: null,
  forceURL: null
};

export default class History {
  static push(url, state) {
    // console.log('href: push', url, 'over', this.getHref());
    if (url === this.getHref() || this.getHref() === '/forceback') {
      // console.log('href: push ignored.');
      return;
    }
    window.history.pushState({state, depth: ++statics.nowDepth}, null, '#' + url);
    listeners.forEach(fn => fn());
  }
  static replace(url, state) {
    // console.log('href: substitute', url, 'for', this.getHref());
    if (url === this.getHref() || this.getHref() === '/forceback') {
      // console.log('href: replace ignored.');
      return;
    }
    window.history.replaceState({state, depth: statics.nowDepth}, null, '#' + url);
    listeners.forEach(fn => fn());
  }
  static force(url) {
    // console.log('href: force', this.getHref(), 'to', url);
    if (url === this.getHref()) return;
    const depth = window.history.state?.depth || 0;
    if (depth) {
      this.go(-depth);
      statics.forceURL = url;
    }
    else
      this.replace(url);
  }

  static listen(listener) {
    listeners.push(listener);
    return () => {
      for (let i in listeners)
        if (listeners[i] === listener)
          listeners.splice(i, 1);
    };
  }
  static go(d) {
    window.history.go(d);
    // console.log("go", d);
  }
  static back() {
    if (statics.nowDepth === 0)
      this.replace('/');
    else
      window.history.back();
    // console.log("goback")
  }
  static forward() {
    window.history.forward();
    // console.log("goforward")
  }

  static getLocation() {
    return document.location;
  }
  static getHref() {
    return document.location.href.match(/#[^#]*$/)?.[0]?.replace('#', '') ?? '/';
  }
  static getId() {
    return this.getHref().match(/[\d]*$/)?.[0] ?? null;
  }

  static init(pageChangeHandler) {
    if (statics.nowDepth != null) return;
    statics.nowDepth = window.history.state?.depth || 0;
    window.addEventListener('hashchange', () => {
      // console.log('href: hash changed to:', this.getHref());
      listeners.forEach(fn => fn());
    });
    return this.listen(() => {
      const depth = window.history.state?.depth || 0;
      if (depth > statics.nowDepth) {
        this.go(statics.nowDepth - depth);
        return;
      }
      else
        statics.nowDepth = depth;
      if (statics.forceURL) {
        const replaceURL = statics.forceURL;
        statics.forceURL = null;
        this.replace(replaceURL);
      }
      else {
        // 触发换页
        pageChangeHandler()
      }
    });
  }
};